import 'dart:collection';

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:integration_test/integration_test.dart';
import 'package:quickblox_sdk/chat/constants.dart';
import 'package:quickblox_sdk/models/qb_dialog.dart';
import 'package:quickblox_sdk/quickblox_sdk.dart';

import '../base/credentials.dart';
import 'utils/custom_data_utils.dart';
import 'utils/dialog_utils.dart';

void main() {
  IntegrationTestWidgetsFlutterBinding.ensureInitialized();

  setUpAll(() async {
    await Credentials.init();
  });

  setUp(() async {
    await Future.delayed(const Duration(seconds: 5), null);

    await Credentials.loginToRest();
    await Credentials.loginToChat();
  });

  tearDown(() async {
    await Credentials.logoutChat();
    await Credentials.logoutRest();
  });

  test("create with correct custom data", () async {
    Map<String, Object> uploadedCustomData = CustomDataUtils.buildCorrectCustomData();
    int uploadedCustomDataSize = uploadedCustomData.length;

    QBDialog? dialog = await QB.chat.createDialog(DialogUtils.buildOccupants(),
        dialogName: DialogUtils.buildDialogName(),
        dialogType: QBChatDialogTypes.CHAT,
        customData: uploadedCustomData);

    int downloadedCustomDataSize = (dialog!.customData! as LinkedHashMap).length;

    expect(uploadedCustomDataSize == downloadedCustomDataSize, true);
  });

  test("create with correct custom data one String field", () async {
    Map<String, Object> uploadedCustomData = CustomDataUtils.buildCorrectCustomData_1String_Field();

    QBDialog? dialog = await QB.chat.createDialog(DialogUtils.buildOccupants(),
        dialogName: DialogUtils.buildDialogName(),
        dialogType: QBChatDialogTypes.CHAT,
        customData: uploadedCustomData);

    String downloadedValue = (dialog?.customData! as LinkedHashMap)["customString"];

    expect(downloadedValue.isNotEmpty, true);
  });

  test("create with empty custom data", () async {
    try {
      await QB.chat.createDialog(DialogUtils.buildOccupants(),
          dialogName: DialogUtils.buildDialogName(),
          dialogType: QBChatDialogTypes.CHAT,
          customData: {});

      fail("Expected: Exception, actual: No thrown Exception");
    } on PlatformException catch (e) {
      expect(e, e);
    }
  });

  test("create with empty custom data but has class name", () async {
    Map<String, Object> uploadedCustomData = CustomDataUtils.buildEmptyCustomDataButHasClassName();

    QBDialog? dialog = await QB.chat.createDialog(DialogUtils.buildOccupants(),
        dialogName: DialogUtils.buildDialogName(),
        dialogType: QBChatDialogTypes.CHAT,
        customData: uploadedCustomData);

    expect(dialog != null, true);
  });

  test("create with wrong custom data", () async {
    Map<String, Object> customData = CustomDataUtils.buildWrongCustomData();

    try {
      await QB.chat.createDialog(DialogUtils.buildOccupants(),
          dialogName: DialogUtils.buildDialogName(),
          dialogType: QBChatDialogTypes.CHAT,
          customData: customData);

      fail("Expected: Exception, actual: No thrown Exception");
    } on PlatformException catch (e) {
      expect(e, e);
    }
  });

  test("create with 3 correct, 1 wrong custom data", () async {
    Map<String, Object> customData = CustomDataUtils.build_3correct_1WrongCustomData();

    try {
      await QB.chat.createDialog(DialogUtils.buildOccupants(),
          dialogName: DialogUtils.buildDialogName(),
          dialogType: QBChatDialogTypes.CHAT,
          customData: customData);

      fail("Expected: Exception, actual: No thrown Exception");
    } on PlatformException catch (e) {
      expect(e, e);
    }
  });
}
