import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:integration_test/integration_test.dart';
import 'package:quickblox_sdk/models/qb_dialog.dart';
import 'package:quickblox_sdk/models/qb_messages_counter.dart';
import 'package:quickblox_sdk/quickblox_sdk.dart';

import '../base/credentials.dart';
import 'utils/unread_messages_utils.dart';

void main() {
  List<QBDialog?> _dialogs = [];

  IntegrationTestWidgetsFlutterBinding.ensureInitialized();

  setUpAll(() async {
    await Credentials.init();
  });

  Future<void> _clearDialogIdsAndLogout() async {
    _dialogs.clear();

    await Credentials.logoutRest();
  }

  setUp(() async {
    await Future.delayed(const Duration(seconds: 5), null);

    await Credentials.loginToRest();

    List<QBDialog?> dialogs = await QB.chat.getDialogs();

    if (dialogs.isEmpty || dialogs.length < 2) {
      _clearDialogIdsAndLogout();

      throw new Exception("The dialogs size doesn't enough for test: " +
          "\nshould be more then 1" +
          "\nactual size: ${dialogs.length}");
    }

    if (UnreadMessagesUtils.calculateCountFromDialogs(dialogs) == 0) {
      await _clearDialogIdsAndLogout();

      throw new Exception("The unread messages size doesn't enough for test:" +
          "\nshould be more then 1" +
          "\nactual size: ${UnreadMessagesUtils.calculateCountFromDialogs(dialogs)}");
    }

    _dialogs = dialogs;
  });

  tearDown(() async {
    await _clearDialogIdsAndLogout();
  });

  test("get with correct array dialog ids", () async {
    final int expectedCount = UnreadMessagesUtils.calculateCountFromDialogs(_dialogs);

    QBMessagesCounter? messageCounter = await QB.chat
        .getTotalUnreadMessagesCount(dialogIds: UnreadMessagesUtils.getIdsFromDialogs(_dialogs));

    int? total = messageCounter?.total;
    List<Map<String, int>>? dialogsList = messageCounter?.dialogs;

    int? actualCount = UnreadMessagesUtils.calculateCountFromListMaps(dialogsList);

    expect(total! > 0, true);
    expect(actualCount, expectedCount);
  });

  test("get with correct one dialog id", () async {
    final int expectedCount = _dialogs[0]!.unreadMessagesCount!;

    QBMessagesCounter? messageCounter =
        await QB.chat.getTotalUnreadMessagesCount(dialogIds: [_dialogs[0]!.id!]);

    int? total = messageCounter?.total;
    List<Map<String, int>>? dialogsList = messageCounter?.dialogs;

    int? actualCount = UnreadMessagesUtils.calculateCountFromListMaps(dialogsList);

    expect(total! > 0, true);
    expect(actualCount, expectedCount);
  });

  test("get with empty dialog ids", () async {
    QBMessagesCounter? messageCounter = await QB.chat.getTotalUnreadMessagesCount(dialogIds: []);

    int? total = messageCounter?.total;
    List<Map<String, int>>? dialogsList = messageCounter?.dialogs;

    int? actualCount = UnreadMessagesUtils.calculateCountFromListMaps(dialogsList);

    expect(total! > 0, true);
    expect(actualCount, null);
  });

  test("get with NULL dialog ids", () async {
    QBMessagesCounter? messageCounter = await QB.chat.getTotalUnreadMessagesCount(dialogIds: null);

    int? total = messageCounter?.total;
    List<Map<String, int>>? dialogsList = messageCounter?.dialogs;

    int? actualCount = UnreadMessagesUtils.calculateCountFromListMaps(dialogsList);

    expect(total! > 0, true);
    expect(actualCount, null);
  });

  test("get with wrong dialog ids", () async {
    List<String> dialogIds = UnreadMessagesUtils.buildWrongDialogIds();

    try {
      await QB.chat.getTotalUnreadMessagesCount(dialogIds: dialogIds);
      fail("Expected: Exception, actual: No thrown Exception");
    } on PlatformException catch (e) {
      expect(e, e);
    }
  });
}
