import 'dart:collection';

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:integration_test/integration_test.dart';
import 'package:quickblox_sdk/chat/constants.dart';
import 'package:quickblox_sdk/models/qb_dialog.dart';
import 'package:quickblox_sdk/quickblox_sdk.dart';

import '../base/credentials.dart';
import 'utils/custom_data_utils.dart';
import 'utils/dialog_utils.dart';

void main() {
  String? _dialogId;

  IntegrationTestWidgetsFlutterBinding.ensureInitialized();

  setUpAll(() async {
    await Credentials.init();
  });

  setUp(() async {
    await Future.delayed(const Duration(seconds: 5), null);

    await Credentials.loginToRest();
    await Credentials.loginToChat();

    QBDialog? dialog = await QB.chat.createDialog(DialogUtils.buildOccupants(),
        dialogName: "Updated ${DialogUtils.buildDialogName()}", dialogType: QBChatDialogTypes.CHAT);
    _dialogId = dialog!.id;
  });

  tearDown(() async {
    await Credentials.logoutChat();
    await Credentials.logoutRest();
    _dialogId = null;
  });

  test("update with correct custom data", () async {
    Map<String, Object> uploadedCustomData = CustomDataUtils.buildUpdatedCorrectCustomData();
    int uploadedCustomDataSize = uploadedCustomData.length;
    String uploadedValue = uploadedCustomData["customString"] as String;

    QBDialog? dialog = await QB.chat.updateDialog(_dialogId!, customData: uploadedCustomData);

    int downloadedCustomDataSize = (dialog!.customData! as LinkedHashMap).length;

    expect(uploadedCustomDataSize == downloadedCustomDataSize, true);

    String downloadedValue = (dialog.customData! as LinkedHashMap)["customString"];

    expect(uploadedValue == downloadedValue, true);
  });

  test("update with correct custom data one String field", () async {
    Map<String, Object> uploadedCustomData =
        CustomDataUtils.buildUpdatedCorrectCustomData_1String_Field();

    String uploadedValue = uploadedCustomData["customString"] as String;

    QBDialog? dialog = await QB.chat.updateDialog(_dialogId!, customData: uploadedCustomData);

    String downloadedValue = (dialog?.customData! as LinkedHashMap)["customString"];

    expect(uploadedValue == downloadedValue, true);
  });

  test("update with empty custom data", () async {
    try {
      await QB.chat.updateDialog(_dialogId!, customData: {});

      fail("Expected: Exception, actual: No thrown Exception");
    } on PlatformException catch (e) {
      expect(e, e);
    }
  });

  test("update with empty custom data but has class name", () async {
    Map<String, Object> customData = CustomDataUtils.buildEmptyCustomDataButHasClassName();

    QBDialog? dialog = await QB.chat.updateDialog(_dialogId!, customData: customData);

    expect(dialog != null, true);
  });

  test("update with wrong custom data", () async {
    Map<String, Object> customData = CustomDataUtils.buildWrongCustomData();
    try {
      await QB.chat.updateDialog(_dialogId!, customData: customData);

      fail("Expected: Exception, actual: No thrown Exception");
    } on PlatformException catch (e) {
      expect(e, e);
    }
  });

  test("update with 3 correct, 1 wrong custom data", () async {
    Map<String, Object> customData = CustomDataUtils.build_3correct_1WrongCustomData();
    try {
      await QB.chat.updateDialog(_dialogId!, customData: customData);

      fail("Expected: Exception, actual: No thrown Exception");
    } on PlatformException catch (e) {
      expect(e, e);
    }
  });
}
